import asyncio

from aioconsole import aprint
from rich import box
from rich.console import Console
from rich.padding import Padding
from rich.table import Table
from rich.traceback import install
from acb.config import Config, adapter_registry
from acb.depends import depends


class RichConsole(Console):
    config: Config = depends()

    def __init__(self) -> None:
        super().__init__()
        if not self.config.deployed:
            install(console=self)

    def _write_buffer(self) -> None:
        with self._lock:
            if self.record and not self._buffer_index:
                with self._record_buffer_lock:
                    self._record_buffer.extend(self._buffer[:])

            if self._buffer_index == 0:
                text = self._render_buffer(self._buffer[:])
                try:
                    asyncio.run(aprint(text))
                except UnicodeEncodeError as error:
                    error.reason = (
                        f"{error.reason}\n*** You may need to add"
                        f" PYTHONIOENCODING=utf-8 to your environment ***"
                    )
                    raise
                self.file.flush()
                del self._buffer[:]


console = RichConsole()


@depends.inject
def display_adapters(config: Config = depends()) -> None:
    if not config.deployed:
        table = Table(
            title="[b][u bright_white]A[/u bright_white][bright_green]synchronous "
            "[u bright_white]C[/u bright_white][bright_green]omponent "
            "[u bright_white]B[/u bright_white][bright_green]ase[/b]",
            show_lines=True,
            box=box.ROUNDED,
            min_width=88,
            border_style="bold blue",
            style="bold white",
        )
        for prop in ("Category", "Name", "Pkg", "Enabled"):
            table.add_column(prop)
        for adapter in adapter_registry.get():
            if adapter.enabled:
                table.add_row(
                    adapter.category,
                    adapter.name,
                    adapter.pkg,
                    str(adapter.enabled),
                    style="bold blue on white",
                )

            else:
                table.add_row(
                    adapter.category, adapter.name, adapter.pkg, str(adapter.enabled)
                )
        table = Padding(table, (2, 4))
        console.print(table)



import hashlib
import typing as t

import dill
from aiocache import BaseCache
from aiocache.serializers import BaseSerializer
from itsdangerous.serializer import Serializer as SecureSerializer
from msgspec import msgpack
from acb.actions.compress import compress, decompress
from acb.config import AdapterBase, Config, Settings, import_adapter
from acb.depends import depends

Logger = import_adapter()


class CacheBaseSettings(Settings):
    default_timeout: int = 86400

    @depends.inject
    def __init__(self, config: Config = depends(), **values: t.Any) -> None:
        super().__init__(**values)
        self.default_timeout = self.default_timeout if config.deployed else 1


class MsgPackSerializer(BaseSerializer):
    def __init__(self, *args: t.Any, use_list: bool = True, **kwargs: t.Any) -> None:
        self.use_list = use_list
        super().__init__(*args, **kwargs)

    def dumps(self, value: t.Any) -> bytes:  # type: ignore
        return compress.brotli(msgpack.encode(value))

    def loads(self, value: bytes | None) -> t.Any:  # type: ignore
        if value is None:
            return None
        return msgpack.decode(decompress.brotli(value))  # type: ignore


class SecurePickleSerializer(BaseSerializer):
    DEFAULT_ENCODING = None

    @depends.inject
    def __init__(
        self, config: Config = depends(), *args: t.Any, **kwargs: t.Any
    ) -> None:
        super().__init__(*args, **kwargs)
        self.serializer = SecureSerializer(
            secret_key=config.app.secret_key.get_secret_value(),
            salt=config.app.secure_salt.get_secret_value(),
            serializer=dill,
            signer_kwargs=dict(digest_method=hashlib.sha256),
        )

    def dumps(self, value: t.Any) -> bytes:  # type: ignore
        return compress.brotli(self.serializer.dumps(value))

    def loads(self, value: bytes | None) -> t.Any:  # type: ignore
        if value is None:
            return None
        return self.serializer.loads(decompress.brotli(value))


class CacheBase(AdapterBase, BaseCache): ...



import typing as t

from aiocache.backends.redis import RedisBackend
from coredis.cache import TrackingCache
from coredis.client import Redis, RedisCluster
from pydantic import SecretStr
from acb.adapters import import_adapter
from acb.config import Config
from acb.debug import debug
from acb.depends import depends
from ._base import CacheBase, CacheBaseSettings, SecurePickleSerializer

Logger = import_adapter()


class CacheSettings(CacheBaseSettings):
    host: SecretStr = SecretStr("127.0.0.1")
    local_host: str = "127.0.0.1"
    port: t.Optional[int] = 6379
    cluster: t.Optional[bool] = False
    connect_timeout: t.Optional[float] = None
    max_connections: t.Optional[int] = None
    health_check_interval: t.Optional[int] = 0

    @depends.inject
    def __init__(self, config: Config = depends(), **values: t.Any) -> None:
        super().__init__(**values)
        self.host = SecretStr(self.local_host) if not config.deployed else self.host


class Cache(CacheBase, RedisBackend):  # type: ignore
    async def _close(self, *args: t.Any, _conn: t.Any = None, **kwargs: t.Any) -> None:
        pass

    async def _clear(
        self, namespace: t.Optional[str] = None, _conn: t.Any = None
    ) -> t.Literal[True]:
        if not namespace:
            pattern = f"{self.config.app.name}:*"
        else:
            pattern = f"{self.config.app.name}:{namespace}:*"
        keys = await self.client.keys(pattern)
        if keys:
            debug(keys)
            for key in keys:
                await self.client.unlink((key,))
        return True

    async def _exists(self, key: str, _conn: t.Any = None) -> bool:
        number = await self.client.exists([key])
        return bool(number)

    async def init(self, *args: t.Any, **kwargs: t.Any) -> t.NoReturn:
        super().__init__(
            serializer=SecurePickleSerializer(),
            namespace=f"{self.config.app.name}:",
            **kwargs,
        )
        redis_kwargs = dict(
            host=self.config.cache.host.get_secret_value(),
            port=self.config.cache.port,
            client_name=self.config.app.name,
            cache=TrackingCache(),
            decode_responses=False,
            connect_timeout=self.config.cache.connect_timeout,
            max_connections=self.config.cache.max_connections,
            health_check_interval=self.config.cache.health_check_interval,
        )
        if self.config.cache.cluster:
            self.logger.info("RedisCluster mode enabled")
            del redis_kwargs["health_check_interval"]
            self.client = RedisCluster(**redis_kwargs)
        else:
            self.client = Redis(**redis_kwargs)


depends.set(Cache)



import typing as t
from abc import ABC, abstractmethod

from acb.config import AdapterBase, Settings


class RequestsBaseSettings(Settings):
    cache_ttl: int = 3600


class RequestsBase(AdapterBase, ABC):
    @abstractmethod
    async def get(self, url: str, timeout: int) -> t.Any:
        raise NotImplementedError

    @abstractmethod
    async def post(self, url: str, data: dict[str, t.Any], timeout: int) -> t.Any:
        raise NotImplementedError

    @abstractmethod
    async def put(self, url: str, data: dict[str, t.Any], timeout: int) -> t.Any:
        raise NotImplementedError

    @abstractmethod
    async def delete(self, url: str, timeout: int) -> t.Any:
        raise NotImplementedError
